@echo off
REM
REM AceQAb.bat %1 %2 [%3]
REM
REM Quality Assurance for ACE files
REM
REM Usage: AceQAb p1 p2 [p3]
REM        p1= ACE formatted filename on acef directory
REM        p2= ENDF formatted filename (full path)
REM        p3= trigger for intermediate file deleting
REM            blank = delete all, anything else keep all
REM
REM Author: D. L. Aldama, IAEA Consultant, Jul. 2005
REM Date: Jul. 2005
REM Updated: Jun. 2008
REM
REM OS: WINDOWS/NT/2000/XP
REM
REM Calling procedure: AceQAall.bat
REM
REM Called codes: ACEQA, ACELST, LINEAR, RECENT, SIGMA1, COMHARD.
REM 
REM Environment variables should be defined on AceQAall.bat procedure. 
REM
   if %1.==. goto errusg
   if %2.==. goto errusg
REM
REM  Skip thermal scattering data files
REM
   for /F "delims=. " %%i in ("%1") do set fnqa=%%i
   if %fnqa%.==lwtr. goto reset
   if %fnqa%.==LWTR. goto reset
   if %fnqa%.==hwtr. goto reset
   if %fnqa%.==HWTR. goto reset
   if %fnqa%.==grph. goto reset
   if %fnqa%.==GRPH. goto reset
REM
REM  Deleting old files
REM
   if exist endf6.lin    del endf6.lin
   if exist endf6.rec    del endf6.rec
   if exist pp02endf.dat del pp02endf.dat
   if exist ace2endf.dat del ace2endf.dat
   if exist MT.DAT       del MT.DAT
   if exist PLOT.CHR     del PLOT.CHR
   if exist PLOT.SYM     del PLOT.SYM
   if exist acelog.log   del acelog.log
   if exist ACELST.MSG   del ACELST.MSG
   if exist ACEQA.INP    del ACEQA.INP
   if exist ACELST.INP   del ACELST.INP
   if exist LINEAR.INP   del LINEAR.INP
   if exist RECENT.INP   del RECENT.INP
   if exist SIGMA1.INP   del SIGMA1.INP
   if exist FIXUP.INP    del FIXUP.INP
   if exist COMPLOT.INP  del COMPLOT.INP
   if exist LINEAR.LST   del LINEAR.LST
   if exist RECENT.LST   del RECENT.LST
   if exist SIGMA1.LST   del SIGMA1.LST
   if exist FIXUP.LST    del FIXUP.LST
   if exist COMPLOT.LST  del COMPLOT.LST
   if exist %adsqad%%fnqa%.QA  del %adsqad%%fnqa%.QA
REM
REM  AceQa
REM
   if not exist %acef%%1 goto errinp
   if not exist %2 goto errinp
   echo %acef%%1  >ACEQA.INP
   echo %2       >>ACEQA.INP
   if not exist ACEQA.INP        goto errqa
   if not exist %daceq%aceqa.exe goto errqa
   %daceq%aceqa<ACEQA.INP
   if not errorlevel 0 goto reset
REM
REM  Acelst
REM
   if not exist ACELST.INP        goto errlst
   if not exist %dacel%acelst.exe goto errlst
   %dacel%acelst<ACELST.INP >ACELST.MSG
   if not errorlevel 0 goto reset
REM
REM  Linear
REM
   if not exist LINEAR.INP        goto errlin
   if not exist %dpp02%linear.exe goto errlin
   %dpp02%linear
   if not errorlevel 0 goto reset
REM
REM  Recent
REM
   if not exist ENDF6.LIN         goto errrec
   if not exist RECENT.INP        goto errrec
   if not exist %dpp02%recent.exe goto errrec
   %dpp02%recent
   if not errorlevel 0 goto reset
REM
REM  Sigma1
REM
   if not exist ENDF6.REC         goto errsig
   if not exist SIGMA1.INP        goto errsig
   if not exist %dpp02%sigma1.exe goto errsig
   %dpp02%sigma1
   if not errorlevel 0 goto reset
REM
REM  Fixup
REM
   if not exist ENDF6.SIG         goto errfix
   if not exist FIXUP.INP         goto errfix
   if not exist %dpp02%fixup.exe  goto errfix
   %dpp02%fixup
   if not errorlevel 0 goto reset
REM
REM  Comhard
REM
   if not exist PP02ENDF.DAT       goto errcom
   if not exist ACE2ENDF.DAT       goto errcom
   if not exist COMPLOT.INP        goto errcom
   if not exist %dcomp%MT.DAT      goto errcom
   if not exist %dcomp%PLOT.CHR    goto errcom
   if not exist %dcomp%PLOT.SYM    goto errcom
   if not exist %dcomp%comhard.exe goto errcom
   if exist PLOT????.ps del PLOT????.ps
   copy %dcomp%MT.DAT
   copy %dcomp%PLOT.CHR
   copy %dcomp%PLOT.SYM
   %dcomp%comhard
   if exist PLOT0001.ps del PLOT0001.ps
   if exist PLOT????.ps copy PLOT????.ps %adsqad%%fnqa%.PS
   if exist PLOT????.ps del PLOT????.ps 
REM
REM   QA log file
REM
   echo ***** QA file for %fnqa% *****    >%adsqad%%fnqa%.QA
   date/T                                >>%adsqad%%fnqa%.QA
   time/T                                >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   echo ***** ACELST messages *****      >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   if exist ACELST.MSG  type ACELST.MSG  >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   echo ***** ACELST logged output ***** >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   if exist ACELOG.LOG  type ACELOG.LOG  >>%adsqad%%fnqa%.QA
   if exist ACELOG.LOG  type ACELOG.LOG   >%adsqad%%fnqa%.TXT
   echo.                                 >>%adsqad%%fnqa%.QA
   echo ***** LINEAR output *****        >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   if exist LINEAR.LST type LINEAR.LST   >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   echo ***** RECENT output *****        >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   if exist RECENT.LST type RECENT.LST   >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   echo ***** SIGMA1 output *****        >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   if exist SIGMA1.LST type SIGMA1.LST   >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   echo ***** FIXUP output *****         >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   if exist FIXUP.LST type FIXUP.LST     >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   echo ***** COMHARD output *****       >>%adsqad%%fnqa%.QA
   echo.                                 >>%adsqad%%fnqa%.QA
   if exist COMPLOT.LST type COMPLOT.LST >>%adsqad%%fnqa%.QA
REM
REM  Done
REM
   if not %3.==. goto reset
   if exist endf6.lin    del endf6.lin
   if exist endf6.rec    del endf6.rec
   if exist endf6.sig    del endf6.sig
   if exist pp02endf.dat del pp02endf.dat
   if exist ace2endf.dat del ace2endf.dat
   if exist MT.DAT       del MT.DAT
   if exist PLOT.CHR     del PLOT.CHR
   if exist PLOT.SYM     del PLOT.SYM
   if exist acelog.log   del acelog.log
   if exist ACELST.MSG   del ACELST.MSG
   if exist ACEQA.INP    del ACEQA.INP
   if exist ACELST.INP   del ACELST.INP
   if exist LINEAR.INP   del LINEAR.INP
   if exist RECENT.INP   del RECENT.INP
   if exist SIGMA1.INP   del SIGMA1.INP
   if exist FIXUP.INP    del FIXUP.INP
   if exist COMPLOT.INP  del COMPLOT.INP
   if exist LINEAR.LST   del LINEAR.LST
   if exist RECENT.LST   del RECENT.LST
   if exist SIGMA1.LST   del SIGMA1.LST
   if exist FIXUP.LST    del FIXUP.LST
   if exist COMPLOT.LST  del COMPLOT.LST
   goto reset
REM
REM  Usage
REM
:errusg
   echo Usage: AceQAb p1 p2 [p3]
   echo        p1= ACE formatted filename
   echo        p2= ENDF formatted filename
   echo        p3= trigger for intermediate file deleting
   echo            blank = delete all, anything else keep all
   goto end
REM
REM  Input data error
REM
:errinp
   echo Input file error
   if not exist %acef%%1 echo %acef%%1 file not found
   if not exist %2 echo %2 file not found
   goto reset
REM
REM
REM
:errqa
   echo ACEQA error
   if not exist ACEQA.INP        echo ACEQA.INP file not found
   if not exist %daceq%aceqa.exe echo %daceq%aceqa.exe not found
   goto reset
REM
REM
REM
:errlst
   echo ACELST error
   if not exist ACELST.INP        echo ACELST.INP file not found
   if not exist %dacel%acelst.exe echo %dacel%acelst.exe not found
   goto reset
REM
REM
REM
:errlin
   echo LINEAR error
   if not exist LINEAR.INP        echo LINEAR.INP file not found
   if not exist %dpp02%linear.exe echo %dpp02%linear.exe not found
   goto reset
REM
REM
REM
:errrec
   if not exist ENDF6.LIN         echo ENDF6.LIN file not found
   if not exist RECENT.INP        echo RECENT.INP file not found
   if not exist %dpp02%recent.exe echo %dpp02%recent.exe not found
   goto reset
REM
REM
REM
:errsig
   if not exist ENDF6.REC         echo ENDF6.REC file not found
   if not exist SIGMA1.INP        echo SIGMA1.INP file not found
   if not exist %dpp02%sigma1.exe echo %dpp02%sigma1.exe not found
   goto reset
REM
REM
REM
:errfix
   if not exist ENDF6.SIG         echo ENDF6.SIG file not found
   if not exist FIXUP.INP         echo FIXUP.INP file not found
   if not exist %dpp02%fixup.exe  echo %dpp02%fixup.exe not found
   goto reset
REM
REM
REM
:errcom
   if not exist PP02ENDF.DAT       echo PP02ENDF.DAT file not found
   if not exist ACE2ENDF.DAT       echo ACE2ENDF.DAT file not found
   if not exist COMPLOT.INP        echo COMPLOT.INP file not found
   if not exist %dcomp%MT.DAT      echo %dcomp%MT.DAT file not found
   if not exist %dcomp%PLOT.CHR    echo %dcomp%PLOT.CHR file not found
   if not exist %dcomp%PLOT.SYM    echo %dcomp%PLOT.SYM file not found
   if not exist %dcomp%comhard.exe echo %dcomp%comhard.exe not found
   goto reset
REM
REM  Reset
REM
:reset
   set fnqa=
REM
REM  End
REM
:end
