       PROGRAM ACELST
C-Title  : ACELST Program
C-Purpose: List reaction data in an ACE file
C-Author : Andrej Trkov, ENEA, Bologna, Italy, 1996.
C-A        Current address: International Atomic Energy Agency
C-A                         Nuclear Data Section
C-A                         Wagramerstrasse 5
C-A                         P.O.Box 100
C-A                         A-1400 Vienna
C-A                         AUSTRIA
C-A                         Tel.: +43 1 2600 21712
C-A                         Fax : +43 1 2600 7
C-A                         E-mail: A.Trkov@iaea.org
C-Version: 1996 Original code
C-V 00/06  A.Trkov: Write cross sections in ENDF format.
C-V 00/09  A.Trkov: Check energy grid to be monotonic increasing.
C-V 00/10  A.Trkov: Write differential cross sections in ENDF format.
C-V 02/02  A.Trkov: Trap corrupted or incomplete files.
C-V 02/12  A.Trkov: - Allow Ace input from XSDIR file.
C-V                 - Implement 9-digit floating point representation.
C-V                 - Implicit double precision.
C-V 05/07  A.Trkov: Write upper energy limit to list file.
C-M
C-M  Manual for Program ACELST
C-M  =========================
C-M
C-M  The program is intended for checking the contents of Ace-
C-M  formatted cross section libraries. It produces a summary of
C-M  contents and optionally converts the data into ENDF format.
C-M  NOTE: At present only the the following are processed
C-M        into ENDF-6 format:
C-M        - Neutron cross sections are formatted in MF3
C-M          representation.
C-M        - Correlated energy/angle distributions stored
C-M          according to Ace Law 67 are formatted in MF6
C-M          representation for neutrons only.
C-M
C-M  Instructions:
C-M  The program is designed to run interactively, requesting
C-M  input from the default input (Unit 5, keyboard) in response
C-M  to the prompts printed on the default output (Unit 6,
C-M  terminal screen). The required input parameters are:
C-M  - Name of the XSDIR file of ACE formatted library.
C-M  - Name of the output list file onto which the contents
C-M    summary is written.
C-M  - Name of the ENDF formatted file. If blank, data conversion
C-M    into ENDF format is skipped.
C-M  - If XSDIR file is specified on input, the next record gives
C-M    the material designation and optionally the partial ACE
C-M    filename.
C-M
C-M  Author's note:
C-M  The program is under development and is offered for
C-M  evaluation and comments "as is". It is intended that
C-M  in the future the functional capability of converting
C-M  the data into ENDF format will be extended to include
C-M  all data that can be processed.
C-
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      PARAMETER   (MXR=4 000 000)
      LOGICAL      EXST
      CHARACTER*80 RC1
      CHARACTER*70 HK
      CHARACTER*66 C66
      CHARACTER*40 BLNK,FLNM,FLN1,FLN2,FLEN,FLAC,PATH,COM
      CHARACTER*10 HZ,HD,HM
      CHARACTER*1  CHDD
      DIMENSION    NXS(16),JXS(32),IZ(16),AW(16),RWO(MXR)
C* Filenames and logical file units
      DATA LIN,LOU,LEN,LXD,LKB,LTT,LAC / 1, 2, 3, 4, 5, 6,-7 /
      DATA BLNK/'                                        '/
     1    ,FLN1/'lib.ace'/
     2    ,FLN2/'ACELST.LOG'/
     3    ,FLEN/'ENDF.DAT'/
     4    ,FLAC/'TMP.ACE'/
C* Boltzman constant
      DATA BK/8.617385E-5/
      LPAT=0
C*
C* Define input parameters - Write banner to terminal
      WRITE(LTT,991) ' ACELST - List contents of an ACE-lib   '
      WRITE(LTT,991) ' ====================================   '
      WRITE(LTT,991)
C* Define the source file
   12 WRITE(LTT,991) ' Default source filename              : ',FLN1
      WRITE(LTT,991) '$          Enter new name to redefine : '
      READ (LKB,991) FLNM
      IF(FLNM.NE.BLNK) FLN1=FLNM
      OPEN(UNIT=LXD,FILE=FLN1,STATUS='OLD',ERR=12)
C* Define the output file
   14 WRITE(LTT,991) ' Default output filename              : ',FLN2
      WRITE(LTT,991) '$          Enter new name to redefine : '
      READ (LKB,991) FLNM
      IF(FLNM.NE.BLNK) FLN2=FLNM
      OPEN (UNIT=LOU,FILE=FLN2,STATUS='UNKNOWN')
C* Define the ENDF file
      WRITE(LTT,991) '$ Enter filename to convert to ENDF-6 : '
      READ (LKB,991) FLNM
      IF(FLNM.EQ.BLNK) GO TO 16
      FLEN=FLNM
      IEN =1
C*
C* Write the banner
   16 WRITE(LOU,991) ' ACELST - List contents of an ACE-lib   '
      WRITE(LOU,991) ' ====================================   '
   17 WRITE(LOU,991)
C*
C* Test if source file is Ace or xsdir
      READ (LXD,908,END=799) RC1
      IF (RC1(1:8).EQ.'        ')GO TO 799
      IXSD=0
      IF(RC1(1:8).EQ.'datapath' .OR. RC1(1:8).EQ.'atomic w') THEN
C* XSDIR file encountered - check for generic pathname
        IF(RC1(1:8).EQ.'datapath') THEN
          I1=9
          CALL STRBND(I1,I2,80,RC1,IER)
          PATH=RC1(I1:I2)
          LPAT=I2-I1+1
        END IF
C* Request selection of material
        IXSD=1
        WRITE(LTT,991) ' XSDIR file encountered on input        '
        GO TO 20
   19   WRITE(LTT,991) '            Please redo - no material : ',FLNM
        REWIND LXD
   20   WRITE(LTT,991) '$      Enter material to be processed : '
        READ (LKB,908) RC1
        I1=1
        CALL STRBND(I1,I2,80,RC1,IER)
        FLNM(1:1+I2-I1)=RC1(I1:I2)
        IF(IER.NE.0) GO TO 800
C* Check for explicit Ace filename
        J1=I2+1
        CALL STRBND(J1,J2,80,RC1,IER)
        IF(IER.EQ.0) THEN
          FLAC=RC1(J1:J2)
        END IF
        IF(LAC.GT.0) CLOSE(UNIT=LAC)
        LAC=ABS(LAC)
C* Find the entry in the XSDIR file
   30   READ (LXD,908,END=19) RC1
        IF(RC1(1:8).EQ.'datapath') THEN
          I1=9
          CALL STRBND(I1,I2,80,RC1,IER)
          PATH=RC1(I1:I2)
          LPAT=I2-I1+1
          GO TO 30
        END IF
        J1=1
        CALL STRBND(J1,J2,80,RC1,IER)
        IF(IER.NE.0) GO TO 800
        IF(RC1(J1:J2).NE.FLNM(I1:I2)) GO TO 30
C* Matching entry found on XSDIR file - dummy read next entry
        J1=J2+1
        CALL STRBND(J1,J2,80,RC1,IER)
C* Extract the filename
        J1=J2+1
        CALL STRBND(J1,J2,80,RC1,IER)
C* Extract the path
        K1=J2+1
        CALL STRBND(K1,K2,80,RC1,IER)
        IF(RC1(K1:K2).NE.'0') THEN
C* Try to open the file as pathname
          FLNM=RC1(K1:K2)
          INQUIRE(FILE=FLNM,EXIST=EXST)
          IF(EXST) GO TO 40
C* Try to open the file as pathname//filename
          FLNM=RC1(K1:K2)//RC1(J1:J2)
          INQUIRE(FILE=FLNM,EXIST=EXST)
          IF(EXST) GO TO 40
        END IF
C* Try to open the file by filename on the default directory
        FLNM=RC1(J1:J2)
        INQUIRE(FILE=FLNM,EXIST=EXST)
        IF(EXST) GO TO 40
C* Try to open the file by filename on the generic directory on XSDIR
        IF(LPAT.GT.0) THEN
C*        Check the last directory delimiter
          DO J=1,LPAT
            IF(PATH(J:J).EQ.'/') CHDD='/'
            IF(PATH(J:J).EQ.'\') CHDD='\'
          END DO
          IF(PATH(LPAT:LPAT).NE.CHDD) THEN
            LPAT=LPAT+1
            PATH(LPAT:LPAT)=CHDD
          END IF
          FLNM=PATH(1:LPAT)//RC1(J1:J2)
          INQUIRE(FILE=FLNM,EXIST=EXST)
          IF(EXST) GO TO 40
        END IF
        WRITE(LTT,991) ' ACELST ERROR - Ace file does not exist ',FLNM
        GO TO 824
C* File exists - open and reassign file units
   40   OPEN (UNIT=LIN,FILE=FLNM,STATUS='OLD')
C* Find the number of records to skip
        J1=K2+1
        CALL STRBND(J1,J2,80,RC1,IER)
        J1=J2+1
        CALL STRBND(J1,J2,80,RC1,IER)
C...         PRINT *,'NSK',J1,J2,IER,RC1(J1:J2)
        READ (RC1(J1:J2), * ) NSK
        NSK=NSK-1
        DO 42 I=1,NSK
          READ (LIN,908)
   42   CONTINUE
C* Find the number of records to copy
        J1=J2+1
        CALL STRBND(J1,J2,80,RC1,IER)
C...          PRINT *,'NCOP',J1,J2,IER,RC1(J1:J2)
        IF(IER.NE.0) THEN
          WRITE(LTT,991) ' Error reading XSDIR record             ',RC1
          GO TO 824
        END IF
        READ (RC1(J1:J2), * ) NSK
        NSK=12+(NSK+3)/4
        OPEN (UNIT=LAC,FILE=FLAC,STATUS='UNKNOWN')
        DO I=1,NSK
          READ (LIN,908) RC1
          WRITE(LAC,908) RC1
        END DO
        REWIND LAC
        LIN=LAC
      ELSE
        BACKSPACE(LXD)
        LIN=LXD
      END IF
C*
C* Begin processing the data file - first value on the file is fixed
  100 READ (LIN,901,ERR=820,END=822) HZ,AW0,TZ,HD
      READ (LIN,902,ERR=820,END=822) HK,HM
      READ (LIN,903,ERR=820,END=822) (IZ(I),AW(I),I=1,16)
      READ (LIN,904,ERR=820,END=822) (NXS(I),I=1,16)
      READ (LIN,904,ERR=820,END=822) (JXS(I),I=1,32)
      ND=NXS(1)
      IF(ND.GT.MXR) THEN
        IF(IEN.EQ.1) WRITE(LTT,*) ' WARNING - Data too long for ENDF'
        IEN=0
      END IF
      NR=MIN(ND,MXR)
      READ (LIN,905,ERR=820,END=822) (RWO(J),J=1,NR)
C* Temperature [K] and upper energy for cross sections
      TK=TZ*1.E6/BK
      JE=JXS(1)
      NP=NXS(3)
      EUMEV=RWO(JE-1+NP)     
C*
C* Write ENDF-6 Files MF1 and MF3 if required
      IF(IEN.NE.1) GO TO 190
      OPEN (UNIT=LEN,FILE=FLEN,STATUS='UNKNOWN')
      READ (HM,974) MAT
C* ENDF file header card
      C66=' Converted form Ace file: '//FLEN
      WRITE(LEN,920) C66, 0, 0, 0
      ZA=NXS(2)
      EU=EUMEV*1.E6
      C66=' '//HK
C* File comments section
      CALL WRIMF1(LEN,MAT,ZA,AW0,EU,TK,C66)
C* Convert energy to eV and check for monotonic ascending order
      E0=0
      DO 180 J=1,NP
      EE=RWO(JE-1+J)*1.E6
      IF(EE.LE.E0)
     1  WRITE(LTT,996) ' ACELST WARNING - x.s. E-grid non-monot.',E0,EE
      RWO(JE-1+J)=EE
      E0=EE
  180 CONTINUE
C* Write total and elastic to the ENDF file
      QI=0.
      QM=0.
      JX=JE+NP
      MT=1
      CALL WRIMF3(LEN,NP,MAT,MT,ZA,AW0,QI,QM,RWO(JE),RWO(JX))
      JX=JX+NP+NP
      MT=2
      CALL WRIMF3(LEN,NP,MAT,MT,ZA,AW0,QI,QM,RWO(JE),RWO(JX))
C* Write summary of contents
  190 WRITE(LOU,902) ' '//HK
      WRITE(LOU,991) '                          Source file : ',FLN1
      WRITE(LOU,992) '             Material ZAID identifier : ',HZ
      WRITE(LOU,992) '                         Processed on : ',HD
      WRITE(LOU,992) '                       MAT identifier : ',HM
      WRITE(LOU,995) '                  Atomic weight ratio : ',AW0
      WRITE(LOU,994) '                      Temperature [K] : ',TK
      WRITE(LOU,994) '             Upper energy limit [MeV] : ',EUMEV
      WRITE(LOU,993) '                    Data table length : ',NXS(1)
      WRITE(LOU,993) '              Number of energy points : ',NXS(3)
      COM =          'Not given                               '
      IF(JXS(12).NE.0)
     1COM =          'Matrix of secondary photon energies     '
      IF(NXS( 6).NE.0)
     1COM =          'Discrete reactions given                '
      WRITE(LOU,991) '               Photon production data : ',COM
      IF(JXS(12).NE.0 .AND. NXS( 6).NE.0)
     1WRITE(LOU,991) '             WARNING - also specified : '
     1              ,'Matrix of secondary photon energies     '
C*
      WRITE(LOU,991)
      WRITE(LOU,991) ' Summary table legend:                  '
      WRITE(LOU,991) '    MT -ENDF reaction MT number         '
      WRITE(LOU,991) '     Q -Reaction Q value                '
      WRITE(LOU,991) '     n -number of outgoing neutrons and '
     1              ,'cordinate system for angular distrib.   '
     1              ,'        (CM=centre-of-mass, LA=lab)     '
      WRITE(LOU,991) ' Angle -Type of distribution (Isotr. or '
     1              ,'Law 44)                                 '
     2              ,'        or "Aen mm" for tabulated distri'
     2              ,'butions at mm energy points             '
      WRITE(LOU,991) ' E-law -Secondary neutron energy distrib'
     1              ,'ution law index list                    '
      WRITE(LOU,991) '    Ty -Photon production cross section '
     1              ,'source (12=MF12, 13=MF13, 16=MF 6)      '
      WRITE(LOU,991)
      WRITE(LOU,991) ' Cross sections (excluding elastic)     '
     1              ,'  Photon Production                     '
      WRITE(LOU,991) '   MT    Q [MeV]     n  Angle E-law     '
     1              ,'  MT Ty  Angle E-law                    '
      WRITE(LOU,991) ' ----------------------------------     '
     1              ,'----------------------------            '
      NTR =NXS(4)
      NRAN=NXS(5)
      NTRP=NXS(6)
      JN  =0
      JP  =0
      KP  =0
      L3  =41
      L4  =L3
C* Prepare constants to print elastic cross section
      TY  =-1
      IT  =1
      MT  =2
      QQ  =0
      LAW =0
      LNW =0
      RC1 =BLNK//BLNK
      GO TO 220
C* Loop over all neutron cross sections (incl.photon prod.)
  200 RC1=BLNK//BLNK
      IF(JN.GE.NTR) GO TO 300
      JN  =JN+1
      MT  =RWO(JXS(3)-1+JN) + 0.1
      QQ  =RWO(JXS(4)-1+JN)
      IF(JN.LE.NRAN) THEN
        TY  =RWO(JXS(5)-1+JN)
      ELSE
        TY  =0
      END IF
      IT  =ABS(TY)          + 0.1
C* Write the section to the ENDF file
      IF(IEN.EQ.1) THEN
        QI=QQ*1.E6
        IF(MT.GE.50 .AND. MT.LE.91) THEN
          QM=0
        ELSE
          QM=QQ*1.E6
        END IF
        JE=JXS(1)
        KJ=RWO(JXS(6)-1+JN) + 0.1
        KK=JXS(7)-1+KJ
        KE=NINT(RWO(KK))
        MP=NINT(RWO(KK+1))
        KX=KK+2
        CALL WRIMF3(LEN,MP,MAT,MT,ZA,AW0,QI,QM,RWO(KE),RWO(KX))
      END IF
      IF(JN.GT.NRAN) GO TO 270
C* Prepare indices for the energy distribution
      LOCC=RWO(JXS(10)-1+JN  )+0.1
      LDIS=    JXS(11)
      LNW =RWO(LDIS-1 +LOCC  )+0.1
      LAW =RWO(LDIS   +LOCC  )+0.1
C* Prepare indices for the angular distribution
  220 LOCB=RWO(JXS( 8)  +JN  )+0.1
      IF(LOCB.GT.0) THEN
        NEA =RWO(JXS( 9)+LOCB-1)+0.1
C*    Check for monotonic ascending energies
        E0  =0
        IM  =0
        DO 260 J=1,NEA
        EE  =RWO(JXS( 9)+LOCB-1+J)
        IF(EE.LE.E0) THEN
        IF(IM.EQ. 0)
     1  WRITE(LTT,993) ' ACELST WARNING - Processing Ang.Dist.MT',MT
        WRITE(LTT,996) '                  E-grid non-monotonic  ',E0,EE
        IM=1
        END IF
        E0=EE
  260   CONTINUE
      END IF
C* Begin filling the list record for this reaction
  270 WRITE(RC1,998) MT,QQ,IT
      IF(IT.EQ.0) THEN
        RC1(21:22)='  '
        GO TO 300
      END IF
C* Cross sections with angle/energy distributions
      IF(TY.GT.0) RC1(19:20)='LA'
      IF(TY.LT.0) RC1(19:20)='CM'
      IF     (LOCB.EQ.0) THEN
        RC1(23:29)=' Isotr.'
      ELSE IF(LOCB.LT.0) THEN
        RC1(23:29)=' Law 44'
      ELSE
        WRITE(RC1(23:29),971) ' Ae',NEA
        IF(RC1(26:26).EQ.' ') RC1(26:26)='n'
      END IF
      L2=31
  280 WRITE(RC1(L2:L2+1),972) LAW
      IF(LNW.EQ.0) GO TO 300
      L2  =L2+2
      LAW =RWO(LDIS+LNW)  +0.1
      LNW =RWO(LDIS-1+LNW)+0.1
      GO TO 280
C*
C* Photon production cross sections
  300 IF(JP.GE.NTRP) GO TO 360
      IF(L2.GE.L3  ) GO TO 360
      JP  =JP+1
      MT  =RWO(JXS(13)-1+JP)+0.1
      LOCA=RWO(JXS(14)-1+JP)+0.1
      MFTY=RWO(JXS(15)-1+LOCA)+0.1
      MT  =MT /1000
      IF(KP.EQ.0) THEN
        WRITE(RC1(L3:L3+7),912) MT,MFTY
        L4  =L3+7
      END IF
      LOCB=RWO(JXS(16)-1+JP)+0.1
      LOCC=RWO(JXS(18)-1+JP)+0.1
      LDIS=    JXS(19)
      NEA =RWO(JXS(17)+LOCB-1)+0.1
      LNW =RWO(LDIS-1 +LOCC  )+0.1
      LAW =RWO(LDIS   +LOCC  )+0.1
      IF     (LOCB.EQ.0) THEN
        RC1(L4:L4+6)=' Isotr.'
      ELSE IF(LOCB.LT.0) THEN
        RC1(L4:L4+6)=' Law 44'
      ELSE
        WRITE(RC1(L4:L4+6),971) ' Ae',NEA
        IF(RC1(L4+3:L4+3).EQ.' ') RC1(L4+3:L4+3)='n'
      END IF
      L4=L4+7
  320 WRITE(RC1(L4:L4+1),972) LAW
      L4  =L4+2
      IF(LNW.EQ.0) GO TO 350
      LAW =RWO(LDIS+LNW)  +0.1
      LNW =RWO(LDIS-1+LNW)+0.1
      GO TO 320
C* Check for more photons from the same MT
  350 IF(JP.GE.NTRP) GO TO 360
      MTT =0.001*RWO(JXS(13)-1+JP+1)+0.1
      IF(MTT.NE.MT) GO TO 360
      RC1(L4:L4)=','
      L4  =L4+1
      L2  =0
      KP  =1
      IF(L4.LE.70) GO TO 300
      L4  =L3+7
      GO TO 362
C* Print a record for neutron and proton reactions
  360 KP  =0
      L4  =L3
  362 WRITE(LOU,908) RC1
      IF(JN.LT.NTR ) GO TO 200
      IF(JP.LT.NTRP) GO TO 200
      IF(IEN.NE.1) GO TO 800
C* All cross section processed - write FEND to ENDF
      C66=BLNK//BLNK(1:26)
      WRITE(LEN,920) C66,MAT, 0, 0
C*
C* Process angular distributions MF 4 into ENDF
      JN  =0
      KN  =0
C* Prepare for processing elastic angular distributions
      MT  =2
      QQ  =0
      LCT =2
      GO TO 422
C* Loop over all other explicitly defined reactions
  420 IF(JN.GE.NRAN) GO TO 480
      JN  =JN+1
      MT  =RWO(JXS(3)-1+JN) + 0.1
      QQ  =RWO(JXS(4)-1+JN)
      TY  =RWO(JXS(5)-1+JN)
      IT  =ABS(TY)          + 0.1
      IF(IT.EQ.0) GO TO 420
C* Cross sections with angular distributions
      IF(TY.GT.0) LCT=1
      IF(TY.LT.0) LCT=2
      LOCC=RWO(JXS(10)-1+JN  )+0.1
      LDIS=    JXS(11)
      LNW =RWO(LDIS-1 +LOCC  )+0.1
      LAW =RWO(LDIS   +LOCC  )+0.1
C* Exclude those represented by correlated distributions
      IF(LAW.EQ.44 .OR. LAW.EQ.67) GO TO 420
  422 JXS9=    JXS( 9)
      LOCB=RWO(JXS( 8)  +JN  )+0.1
      IF(LOCB.LT.0) THEN
        PRINT *,' Law 44 not processed at present for MT',MT
      ELSE IF(LOCB.GT.0) THEN
C* Write the section to the ENDF file
        QI=QQ*1.E6
        QM=QQ*1.E6
        CALL WRIMF4(LEN,MAT,MT,ZA,AW0,QI,QM,JXS9,LOCB,RWO,LCT,IER)
        IF(IER.EQ.0) KN=KN+1
      END IF
      IF(JN.LT.NRAN) GO TO 420
  480 IF(KN.LE.0) GO TO 600
C* All angular distributions processed - write FEND to ENDF
      C66=BLNK//BLNK(1:26)
      WRITE(LEN,920) C66,MAT, 0, 0
C*
C* Process double differential cross sections into ENDF
  600 JN  =0
      KN  =0
C* Loop over all reactions
  620 IF(JN.GE.NRAN) GO TO 680
      JN  =JN+1
      MT  =RWO(JXS(3)-1+JN) + 0.1
      QQ  =RWO(JXS(4)-1+JN)
      TY  =RWO(JXS(5)-1+JN)
      IT  =ABS(TY)          + 0.1
      IF(IT.EQ.0) GO TO 620
C* Cross sections with angle/energy distributions
      IF(TY.GT.0) LCT=1
      IF(TY.LT.0) LCT=2
      LOCC=RWO(JXS(10)-1+JN  )+0.1
      LDIS=    JXS(11)
      LNW =RWO(LDIS-1 +LOCC  )+0.1
      LAW =RWO(LDIS   +LOCC  )+0.1
      IDAT=RWO(LDIS+1 +LOCC  )+0.1
      LDAT=LDIS+IDAT-1
      IF(LNW.NE.0) THEN
         PRINT *,'ACELST WARNING - More than one range for MF/MT',6,MT
C...         STOP
         GO TO 620
      END IF
      IF(LAW.LT.40) GO TO 620
C* Write the section to the ENDF file
      QI=QQ*1.E6
      QM=QQ*1.E6
      LOFF=LDIS-LDAT
      CALL WRIMF6(LEN,MAT,MT,ZA,AW0,QI,QM,RWO(LDAT),RWO
     1           ,LOFF,IT,LCT,LAW,IER)
      IF(IER.EQ.0) KN=KN+1
      IF(JN.LT.NRAN) GO TO 620
  680 IF(KN.LE.0) GO TO 800
C* All cross section processed - write FEND to ENDF
      C66=BLNK//BLNK(1:26)
      WRITE(LEN,920) C66,MAT, 0, 0
C*
C* End of file processing
  800 IF(IEN.EQ.1) THEN
        C66=BLNK//BLNK(1:26)
        WRITE(LEN,920) C66,  0, 0, 0
        WRITE(LEN,920) C66, -1, 0, 0
      END IF
      IF (IXSD.EQ.0) GO TO 17
  799 STOP 'ACELST Completed'
C*
C* Error traps
  820 WRITE(LTT,991) ' Ace file corrupted - Process terminated'
      STOP 'ACELST ERROR reading Ace file'
  822 WRITE(LTT,991) ' Ace file truncated - Process terminated'
  824 STOP 'ACELST ERROR reading Ace file'
C*
  901 FORMAT(A10,2E12.0,1X,A10)
  902 FORMAT(A70,A10)
  903 FORMAT(4(I7,F11.0))
  904 FORMAT(8I9)
  905 FORMAT(4F20.0)
  908 FORMAT(A80)
  912 FORMAT(I4,I3)
  920 FORMAT(A66,I4,I2,I3,I5)
  971 FORMAT(A3,I4)
  972 FORMAT(I2)
  974 FORMAT(6X,I4)
  991 FORMAT(2A40)
  992 FORMAT(A40,A10)
  993 FORMAT(A40,I10)
  994 FORMAT(A40,F12.1)
  995 FORMAT(A40,F10.5)
  996 FORMAT(A40,1P,2E16.9)
  998 FORMAT(I5,1P,E12.4,I5)
      END
      SUBROUTINE STRBND(I1,I2,LR,REC,IER)
C-Title  : Subroutine STRBND
C-Purpose: Fins the next non-blank range in string REC
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      CHARACTER*80 REC
      IER=0
C* Skip leading blanks
      I1=I1-1
   22 I1=I1+1
      IF(I1.GT.LR) GO TO 82
      IF(REC(I1:I1).EQ.' ') GO TO 22
C* Find the last contiguous non-blank character
      I2=I1-1
   24 I2=I2+1
      IF(I2+1.LE.LR .AND. REC(I2+1:I2+1).NE.' ') GO TO 24
      RETURN
C* No non-blank string found
   82 IER=1
      RETURN
      END
      SUBROUTINE WRIMF1(LEN,MAT,ZA,AWR,EU,TEM,C66)
C-Title  : Subroutine WRIMF1
C-Purpose: Write ENDF comments section
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      CHARACTER*66 C66
      CHARACTER*33 C33
      DATA C33/'                                 '/
C*
      MF=1
      MT=451
      LRP=-1
      NLIB=0
      NSUB=10
      WRITE(LEN,822)  ZA,AWR,LRP,  0,NLIB, 0,MAT,MF,MT
      WRITE(LEN,822)  0., 0.,  0,  0,   0, 6,MAT,MF,MT
      WRITE(LEN,822)  1., EU,  0,  0,NSUB, 0,MAT,MF,MT
      WRITE(LEN,822) TEM, 0.,  2,  0,   1, 0,MAT,MF,MT
      WRITE(LEN,820) C66,                MAT,MF,MT
      WRITE(LEN,820) C33//C33,           MAT,MF, 0
      WRITE(LEN,820) C33//C33,           MAT, 0, 0
  820 FORMAT(A66,I4,I2,I3,I5)
  822 FORMAT(1P,2E11.5E1,4I11,I4,I2,I3,I5)
      END
      SUBROUTINE WRIMF3(LEN,NP,MAT,MT,ZA,AWR,QI,QM,EN,XS)
C-Title  : Subroutine WRIMF3
C-Purpose: Write cross sections in ENDF format
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      CHARACTER*11 REC(6)
      DIMENSION    EN(1),XS(1)
C*
      MF=3
      WRITE(LEN,822) ZA,AWR, 0, 0, 0, 0,MAT,MF,MT
      WRITE(LEN,822) QM, QI, 0, 0, 1,NP,MAT,MF,MT
      WRITE(LEN,824) NP,  2, 0, 0, 0, 0,MAT,MF,MT
      IP=0
C* Initialize record
   20 IR=0
      DO 22 J=1,6
      REC(J)='          '
   22 CONTINUE
      IF(IP.GE.NP) GO TO 80
C* Pack an energy-cross section pair to output field
   40 IP=IP+1
      CALL C11PCK(EN(IP),REC(IR+1))
      CALL C11PCK(XS(IP),REC(IR+2))
      IR=IR+2
      IF(IR.GE. 6) GO TO 60
      IF(IP.LT.NP) GO TO 40
C* Write a record to output
   60 WRITE(LEN,826) (REC(J),J=1,6),MAT,MF,MT
      GO TO 20
C* Terminate section
   80 WRITE(LEN,826) (REC(J),J=1,6),MAT,MF, 0
      RETURN
C*
  822 FORMAT(1P,2E11.5E1,4I11,I4,I2,I3,I5)
  824 FORMAT(6I11,I4,I2,I3,I5)
  826 FORMAT(6A11,I4,I2,I3,I5)
      END
      SUBROUTINE WRIMF4(LEN,MAT,MT,ZA,AWR,QI,QM,JXS9,LOCB
     1                 ,RWO,LCT,IER)
C-Title  : Subroutine WRIMF4
C-Purpose: Format angular distribution data in ENDF-6
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      PARAMETER   (NMU=33)
      CHARACTER*11 REC(6)
      DIMENSION    RWO(1)
C*
      IER=0
      MF =4
C* Consider the case of isotropic distributions at all energies
      IF(LOCB.EQ.0) THEN
        LVT=0
        LTT=0
        LI =1
        WRITE(LEN,822) ZA ,AWR,LVT,LTT,  0,  0,MAT,MF,MT
        WRITE(LEN,822) 0. ,AWR, LI,LCT,  0,  0,MAT,MF,MT
        GO TO 80
      END IF
C* Number of incident energy locations and location
      NEA =RWO(JXS9+LOCB-1)+0.1
      LEA =    JXS9+LOCB
C* File header information
      LTT=2
      LVT=0
      LI =0
      NR =1
      WRITE(LEN,822) ZA ,AWR,LVT,LTT,  0,  0,MAT,MF,MT
      WRITE(LEN,822) 0. ,AWR, LI,LCT,  0,  0,MAT,MF,MT
      WRITE(LEN,822) 0. , 0.,  0,  0, NR,NEA,MAT,MF,MT
      WRITE(LEN,824) NEA,  2,  0,  0,  0,  0,MAT,MF,MT
C* Loop over all incident energies
      DO 200 I=1,NEA
      EE=RWO(LEA    -1+I)*1.E6
      LP=RWO(LEA+NEA-1+I)+0.1
      IF(LP.GT.0) THEN
        LP=LP+JXS9
        NP=NMU
      ELSE
        NP=2
      END IF
C* Define histogram interpolation
      WRITE(LEN,822) 0. , EE,  0,  0, NR, NP,MAT,MF,MT
      WRITE(LEN,824) NP ,  1,  0,  0,  0,  0,MAT,MF,MT
C* Write the angular distribution to the ENDF file
      IP=0
C* Initialize record
   20 IR=0
      DO 22 J=1,6
      REC(J)='          '
   22 CONTINUE
      IF(IP.GE.NP) GO TO 200
C* Pack cosine/distribution pairs to output field
   40 IP=IP+1
C* Calculate cosines and probabilities from equiprobable cosine bins
      IF(LP.GT.0) THEN
        AMU=RWO(LP-2+IP)
        IF(IP.LT.NP) THEN
          AM1=RWO(LP-1+IP)
          FMU=1/((NP-1)*(AM1-AMU))
        ELSE
          FMU=0.
        END IF
      ELSE
        AMU=IP*2-3
        FMU=0.5
      END IF
      CALL C11PCK(AMU,REC(IR+1))
      CALL C11PCK(FMU,REC(IR+2))
      IR=IR+2
      IF(IR.GE. 6) GO TO 60
      IF(IP.LT.NP) GO TO 40
C* Write a record to output
   60 WRITE(LEN,826) (REC(J),J=1,6),MAT,MF,MT
      GO TO 20
  200 CONTINUE
C*
C* Terminate section
   80 WRITE(LEN,826) (REC(J),J=1,6),MAT,MF, 0
      RETURN
C*
  822 FORMAT(1P,2E11.5E1,4I11,I4,I2,I3,I5)
  824 FORMAT(6I11,I4,I2,I3,I5)
  826 FORMAT(6A11,I4,I2,I3,I5)
      END
      SUBROUTINE WRIMF6(LEN,MAT,MT,ZA,AWR,QI,QM,RWO,YLD
     1                 ,LDIS,IT,LCT,LAW,IER)
C-Title  : Subroutine WRIMF6
C-Purpose: Write ENDF file MF6 in Law 7 representation
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      PARAMETER   (MXMU=33)
      CHARACTER*11 REC(6)
      DIMENSION    RWO(1)
      DIMENSION    AMU(MXMU),PMU(MXMU)
C*
      DD=2.0/(MXMU-1)
      DO 12 I=2,MXMU
      AMU(I)=-1+(I-2)*DD
   12 CONTINUE
      AMU(MXMU)=1
C*
      IER=0
      IF(LAW.NE.67) THEN
        PRINT *,' Formatting of MF6 not coded for MT',MT
     1         ,' Law',LAW
        IER=1
        RETURN
      END IF
      NR = RWO(1)+0.1
      NE = RWO(2+2*NR)
C*
      MF =6
      NK =1
      LIP=0
      LWE=7
      ZAP=1.
C*
C* Neutron multiplicities
      IF(IT.LT.100) THEN
        NPN=2
        TY =IT
      ELSE
        PRINT *,' Formatting of MF6 not coded for MT'
     1         ,MT,'  Multipl.flag',IT
        IER=1
        RETURN
      END IF
      WRITE(LEN,822) ZA ,AWR,  0,LCT,NK,  0,MAT,MF,MT
      WRITE(LEN,822) ZAP, 1.,LIP,LWE, 1,NPN,MAT,MF,MT
      WRITE(LEN,824) NPN,  2,  0,  0, 0,  0,MAT,MF,MT
      E1=RWO(2+2*NR+ 1)*1.E6
      E2=RWO(2+2*NR+NE)*1.E6
      CALL C11PCK(E1,REC(1))
      CALL C11PCK(TY,REC(2))
      CALL C11PCK(E2,REC(3))
      CALL C11PCK(TY,REC(4))
      REC(5)='           '
      REC(6)='           '
      WRITE(LEN,826) (REC(J),J=1,6),MAT,MF,MT
C*
C* Incident particle energy interpolation TAB2 record
      IF(NR.EQ.0) THEN
        WRITE(LEN,822) 0. , 0.,  0,  0,  1, NE,MAT,MF,MT
        WRITE(LEN,824) NE ,  2,  0,  0,  0,  0,MAT,MF,MT
      ELSE
        PRINT *,' Formatting of energy/angle distrib. not coded for MT'
     1         ,MT,'  NR>0',NR
        IER=1
        RETURN
      END IF
C*
C* Loop over incident particle energies
      DO 200 IE=1,NE
      EE = RWO(2+2*NR+IE)*1.E6
      LL = RWO(2+2*NR+IE+NE)+0.1
      LL = LL+LDIS
C* Interpolation law and No. of cosines
      NMU= RWO(LL+1)+0.1
      INU= RWO(LL  )+0.1
      IF     (LAW.EQ.44) THEN
        KMU=MXMU
        INU=2
      ELSE IF(LAW.EQ.67) THEN
        KMU=NMU
      END IF
      WRITE(LEN,822) 0. , EE,  0,  0,  1,KMU,MAT,MF,MT
      WRITE(LEN,824) KMU,INU,  0,  0,  0,  0,MAT,MF,MT
C*
C* Loop over all cosines
      DO 100 M=1,KMU
      IF     (LAW.EQ.44) THEN
        XMU= AMU(M)
        LMU= RWO(LL)
      ELSE IF(LAW.EQ.67) THEN
        XMU= RWO(LL+1+M)
        LMU= RWO(LL+1+M+NMU)
        LMU= LMU+LDIS
      END IF
      NEP= RWO(LMU+1)+0.1
      INP= RWO(LMU  )+0.1
      WRITE(LEN,822) 0. ,XMU,  0,  0,  1,NEP,MAT,MF,MT
      WRITE(LEN,824) NEP,INP,  0,  0,  0,  0,MAT,MF,MT
C* Initialize record
      IP=0
   20 IR=0
      DO 22 K=1,6
      REC(K)='          '
   22 CONTINUE
      IF(IP.GE.NEP) GO TO 100
C* Pack an energy/distribution pair to output field
   40 IP=IP+1
      EE=1.E+6*RWO(LMU+1+IP    )
      IF     (LAW.EQ.44) THEN
        PP=RWO(LMU+1+IP+NEP)
        RR=RWO(LMU+1+IP+NEP*3)
        AA=RWO(LMU+1+IP+NEP*4)
        PP=PP*AA*(COSH(AA*XMU)+RR*SINH(AA*XMU))/(2*SINH(AA))
      ELSE IF(LAW.EQ.67) THEN
        PP=.5E-6*RWO(LMU+1+IP+NEP)
      END IF
      CALL C11PCK(EE,REC(IR+1))
      CALL C11PCK(PP,REC(IR+2))
      IR=IR+2
      IF(IR.GE.  6) GO TO 60
      IF(IP.LT.NEP) GO TO 40
C* Write a record to output
   60 WRITE(LEN,826) (REC(K),K=1,6),MAT,MF,MT
      GO TO 20
C* End loop over cosines
  100 CONTINUE
C* End loop over energies
  200 CONTINUE
C* Terminate section
      WRITE(LEN,826) (REC(J),J=1,6),MAT,MF, 0
      RETURN
C*
  822 FORMAT(1P,2E11.5E1,4I11,I4,I2,I3,I5)
  824 FORMAT(6I11,I4,I2,I3,I5)
  826 FORMAT(6A11,I4,I2,I3,I5)
      END
      SUBROUTINE C11PCK(PAR,C11)
C-Title  : C11PCK Subroutine
C-Purpose: Pack real exponential values to 11 characters
C-Version: 1996 original code
C-V  02/12 Implement 9-digit floating point representation
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      CHARACTER*11 C11
      CHARACTER*13 DMY
      APA=ABS(PAR)
      IF     (APA.LE.0.099999949 .OR. APA.GE. 9999999999.49) THEN
        WRITE(DMY,'(1P,E13.6)') PAR
        L=2
        IF(DMY(12:12).EQ.'0' .OR. DMY(12:12).EQ.' ') THEN
          DMY(12:12)=DMY(13:13)
          L=1
        END IF
        C11=DMY
        C11(11-L:11)=DMY(11:11+L)
        RETURN
      ELSE IF(APA.LT. 9.9999999949) THEN
        WRITE(C11,'(F11.8)') PAR
        RETURN
      ELSE IF(APA.LT. 99.999999949) THEN
        WRITE(C11,'(F11.7)') PAR
        RETURN
      ELSE IF(APA.LT. 999.99999949) THEN
        WRITE(C11,'(F11.6)') PAR
        RETURN
      ELSE IF(APA.LT. 9999.9999949) THEN
        WRITE(C11,'(F11.5)') PAR
        RETURN
      ELSE IF(APA.LT. 99999.999949) THEN
        WRITE(C11,'(F11.4)') PAR
        RETURN
      ELSE IF(APA.LT. 999999.99949) THEN
        WRITE(C11,'(F11.3)') PAR
        RETURN
      ELSE IF(APA.LT. 9999999.9949) THEN
        WRITE(C11,'(F11.2)') PAR
        RETURN
      ELSE IF(APA.LT. 99999999.949) THEN
        WRITE(C11,'(F11.1)') PAR
        RETURN
      ELSE IF(APA.LT. 9999999999.49) THEN
        WRITE(C11,'(I11)') NINT(PAR)
        RETURN
      ELSE
        PRINT *,' ERROR converting for 9-digit real',apa
        STOP 'ACELST ERROR - converting to 9-digit real'
      END IF
      END
